package com.riiiver.sample.qcc5100.di

import android.content.Context
import com.riiiver.sample.qcc5100.util.GetAppInfoUseCase
import com.riiiver.sample.qcc5100.util.GetAppInfoUseCaseImpl
import com.riiiver.sample.qcc5100.util.riiiver.RiiiverSdkClient
import com.riiiver.sample.qcc5100.util.ble.BleCentralClient
import com.riiiver.sample.qcc5100.util.ble.BleGattCallback
import com.riiiver.sample.qcc5100.util.ble.BleScanCallback
import com.riiiver.sample.qcc5100.util.ble.IBleCentralClient
import com.riiiver.sample.qcc5100.util.ble.IBleGattCallback
import com.riiiver.sample.qcc5100.util.ble.IBleScanCallback
import com.riiiver.sample.qcc5100.util.riiiver.IRiiiverSdkClient
import com.riiiver.sdk.RiiiverSdkManager
import dagger.Binds
import dagger.Module
import dagger.Provides
import dagger.hilt.InstallIn
import dagger.hilt.android.qualifiers.ApplicationContext
import dagger.hilt.components.SingletonComponent
import kotlinx.coroutines.CoroutineScope
import javax.inject.Singleton

@Module
@InstallIn(SingletonComponent::class)
object AppModule {
    @Provides
    @Singleton
    fun provideRiiiverSdkClient(
        @ApplicationContext context: Context,
        @ApplicationScope externalScope: CoroutineScope
    ) = RiiiverSdkClient(context, externalScope)

    @Provides
    @Singleton
    fun provideBleClient(
        @ApplicationContext context: Context,
        @ApplicationScope externalScope: CoroutineScope
    ) = BleCentralClient(context, externalScope, provideBleScanCallback(), provideBleGattCallback())

    @Provides
    @Singleton
    fun provideBleScanCallback() = BleScanCallback()

    @Provides
    @Singleton
    fun provideBleGattCallback() = BleGattCallback()

    @Provides
    fun provideGetAppInfoUseCase(
        @ApplicationContext context: Context
    ) = GetAppInfoUseCaseImpl(context)
}

@Module
@InstallIn(SingletonComponent::class)
interface IBleBindModules {
    @Binds
    fun bindRiiiverSdkClient(riiiverSdkClient: RiiiverSdkClient): IRiiiverSdkClient
    @Binds
    fun bindBleCentralClient(bleCentralClient: BleCentralClient): IBleCentralClient
    @Binds
    fun bindBleScanCallback(bleScanCallback: BleScanCallback): IBleScanCallback
    @Binds
    fun bindBleGattCallback(bleGattCallback: BleGattCallback): IBleGattCallback
    @Binds
    fun bindGetAppInfoUseCase(getAppInfoUseCaseImpl: GetAppInfoUseCaseImpl): GetAppInfoUseCase

}