package com.riiiver.sample.qcc5100.router

import android.app.Application
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.core.content.ContextCompat.getString
import androidx.navigation.NavHostController
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.navigation.navDeepLink
import com.riiiver.sample.qcc5100.MyApplication
import com.riiiver.sample.qcc5100.R
import com.riiiver.sample.qcc5100.features.device_search.DeviceSearchScreen
import com.riiiver.sample.qcc5100.features.device_settings.DeviceSettingsScreen
import com.riiiver.sample.qcc5100.features.riiiver_settings.RiiiverSettingsScreen

@Composable
fun NavGraph(
    modifier: Modifier = Modifier,
    navController: NavHostController = rememberNavController(),
    startDestination: String = Destinations.DEVICE_SEARCH_ROUTE,
    navActions: NavigationActions = remember(navController) {
        NavigationActions(navController)
    }
) {
    val loginScheme: String = stringResource(id = R.string.login_scheme)

    NavHost(
        navController = navController,
        startDestination = startDestination,
        modifier = modifier,
    ) {
        composable(
            Destinations.DEVICE_SEARCH_ROUTE,
        ) {
            DeviceSearchScreen(
                onDeviceClick = { navActions.navigateToDeviceSettings()}
            )
        }
        composable(
            Destinations.DEVICE_SETTINGS_ROUTE,
        ) {
            DeviceSettingsScreen(
                onBack = { navController.popBackStack() },
                onClickSettings = { navActions.navigateToRiiiverSettings() }
            )
        }
        composable(
            Destinations.RIIIVER_SETTINGS_ROUTE,
            deepLinks = listOf(navDeepLink { uriPattern = "$loginScheme:///weblogin?token={token}" })
        ) {
            RiiiverSettingsScreen(
                onBack = { navController.popBackStack() },
                webToken = it.arguments?.getString("token")
            )
        }
    }
}