package com.riiiver.sample.qcc5100.router

import androidx.navigation.NavGraph.Companion.findStartDestination
import androidx.navigation.NavHostController
import com.riiiver.sample.qcc5100.router.Screens.DEVICE_SEARCH_SCREEN
import com.riiiver.sample.qcc5100.router.Screens.DEVICE_SETTINGS_SCREEN
import com.riiiver.sample.qcc5100.router.Screens.RIIIVER_SETTINGS_SCREEN

/**
 * Screens used in [Destinations]
 */
private object Screens {
    const val DEVICE_SEARCH_SCREEN = "device_search"
    const val DEVICE_SETTINGS_SCREEN = "device_settings"
    const val RIIIVER_SETTINGS_SCREEN = "riiiver_settings"
}

/**
 * Arguments used in [Destinations] routes
 */
object DestinationsArgs {
    const val DEVICE_ID_ARG = "deviceId"
}

/**
 * Destinations used in the [MyActivity]
 */
object Destinations {
    const val DEVICE_SEARCH_ROUTE = DEVICE_SEARCH_SCREEN
    const val DEVICE_SETTINGS_ROUTE = DEVICE_SETTINGS_SCREEN
    const val RIIIVER_SETTINGS_ROUTE = RIIIVER_SETTINGS_SCREEN
}

/**
 * Models the navigation actions in the app.
 */
class NavigationActions(private val navController: NavHostController) {
    fun navigateToDeviceSearch() {
        navController.navigate(Destinations.DEVICE_SEARCH_ROUTE) {
            // Pop up to the start destination of the graph to
            // avoid building up a large stack of destinations
            // on the back stack as users select items
            popUpTo(navController.graph.findStartDestination().id) {
                saveState = true
            }
            // Avoid multiple copies of the same destination when
            // reselecting the same item
            launchSingleTop = true
            // Restore state when reselecting a previously selected item
            restoreState = true
        }
    }
    fun navigateToDeviceSettings() {
        navController.navigate("$DEVICE_SETTINGS_SCREEN")
    }

    fun navigateToRiiiverSettings() {
        navController.navigate("$RIIIVER_SETTINGS_SCREEN")
    }
}