package com.riiiver.sample.qcc5100.router

import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.material.Icon
import androidx.compose.material.IconButton
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Surface
import androidx.compose.material.Text
import androidx.compose.material.TopAppBar
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.Delete
import androidx.compose.material.icons.filled.Info
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.tooling.preview.Preview
import com.riiiver.sample.qcc5100.R

@Composable
fun DeviceSearchNavigationBar(onClickInfo: () -> Unit, onClickDelete: () -> Unit) {
    TopAppBar(
        title = { Text(text = stringResource(id = R.string.device_search)) },
        actions = {
            IconButton(onClick = onClickInfo) {
                Icon(Icons.Filled.Info, stringResource(id = R.string.app_version_info))
            }
            IconButton(onClick = onClickDelete) {
                Icon(Icons.Filled.Delete, stringResource(id = R.string.clear_device))
            }
        },
        modifier = Modifier.fillMaxWidth(),
    )
}

@Composable
fun DeviceSettingsNavigationBar(onBack: () -> Unit) {
    TopAppBar(
        title = { Text(text = stringResource(id = R.string.device_settings)) },
        navigationIcon = {
            IconButton(onClick = onBack) {
                Icon(Icons.AutoMirrored.Filled.ArrowBack, stringResource(id = R.string.previous))
            }
        },
        modifier = Modifier.fillMaxWidth(),
    )
}

@Composable
fun RiiiverSettingsNavigationBar(onBack: () -> Unit) {
    TopAppBar(
        title = { Text(text = stringResource(id = R.string.riiiver_settings)) },
        navigationIcon = {
            IconButton(onClick = onBack) {
                Icon(
                    Icons.AutoMirrored.Filled.ArrowBack,
                    stringResource(id = R.string.previous),
                )
            }
        },
        modifier = Modifier.fillMaxWidth(),
    )
}

@Composable
fun RiiiverLoginNavigationBar(onBack: () -> Unit) {
    TopAppBar(
        title = { Text(text = stringResource(id = R.string.riiiver_login)) },
        navigationIcon = {
            IconButton(onClick = onBack) {
                Icon(
                    Icons.AutoMirrored.Filled.ArrowBack,
                    stringResource(id = R.string.previous),
                )
            }
        },
        modifier = Modifier.fillMaxWidth(),
    )
}

@Preview
@Composable
private fun DeviceSearchNavigationBarPreview() {
    MaterialTheme {
        Surface {
            DeviceSearchNavigationBar({}, {})
        }
    }
}

@Preview
@Composable
private fun DeviceSettingsNavigationBarPreview() {
    MaterialTheme {
        Surface {
            DeviceSettingsNavigationBar {}
        }
    }
}

@Preview
@Composable
private fun RiiiverSettingsNavigationBarPreview() {
    MaterialTheme {
        Surface {
            RiiiverSettingsNavigationBar {}
        }
    }
}

@Preview
@Composable
private fun RiiiverLoginNavigationBarPreview() {
    MaterialTheme {
        Surface {
            RiiiverLoginNavigationBar {}
        }
    }
}