package com.riiiver.sample.qcc5100.util

import android.content.Context
import android.content.pm.PackageInfo
import android.content.pm.PackageManager
import dagger.hilt.android.qualifiers.ApplicationContext
import javax.inject.Inject

interface GetAppInfoUseCase {
    fun versionName(): String
    fun versionCode(): Int
}

class GetAppInfoUseCaseImpl @Inject constructor(
    @ApplicationContext private val context: Context
): GetAppInfoUseCase {
    override fun versionName(): String {
        try {
            val info: PackageInfo = context.packageManager.getPackageInfo(context.packageName, 0)
            return info.versionName
        } catch (e: PackageManager.NameNotFoundException) {
            e.printStackTrace()
            return "x.x.x"
        }
    }

    override fun versionCode(): Int {
        try {
            val info: PackageInfo = context.packageManager.getPackageInfo(context.packageName, 0)
            return info.longVersionCode.toInt()
        } catch (e: PackageManager.NameNotFoundException) {
            e.printStackTrace()
            return 19700101
        }
    }
}