package com.riiiver.sample.qcc5100.util.ble

class BleApi {
    data class StartScan(
        override val type: Operation = Operation.START_SCAN
    ) : BleOperation

    data class StopScan(
        override val type: Operation = Operation.STOP_SCAN
    ) : BleOperation

    data class Connect(
        override val type: Operation = Operation.CONNECT,
        override val deviceAddress: String
    ) : BleOperation

    data class Disconnect(
        override val type: Operation = Operation.DISCONNECT
    ) : BleOperation

    data class DiscoverServices(
        override val type: Operation = Operation.DISCOVER_SERVICES
    ) : BleOperation

    data class SetNotification(
        override val type: Operation = Operation.SET_NOTIFICATION
    ) : BleOperation

    data class SendResultButton1(
        override val type: Operation = Operation.WRITE_CHARACTERISTIC,
        override val command: CommandReceiverCommand = CommandReceiverCommand.BUTTON_1_RESULT,
        override val message: String
    ) : BleOperation

    data class SendResultButton2(
        override val type: Operation = Operation.WRITE_CHARACTERISTIC,
        override val command: CommandReceiverCommand = CommandReceiverCommand.BUTTON_2_RESULT,
        override val message: String
    ) : BleOperation

    data class SendDisplayData(
        override val type: Operation = Operation.WRITE_CHARACTERISTIC,
        override val command: CommandReceiverCommand = CommandReceiverCommand.DISPLAY,
        override val message: String
    ) : BleOperation

    data class ClearResults(
        override val type: Operation = Operation.CLEAR_RESULTS
    ) : BleOperation
}

interface BleOperation {
    val type: Operation
    val command: CommandReceiverCommand?
        get() = null
    val deviceAddress: String?
        get() = null
    val message: String?
        get() = null
}

enum class Operation {
    START_SCAN,
    STOP_SCAN,
    CONNECT,
    DISCONNECT,
    DISCOVER_SERVICES,
    SET_NOTIFICATION,
    WRITE_CHARACTERISTIC,
    CLEAR_RESULTS;
}

enum class CommandReceiverCommand(val rawValue: Int) {
    BUTTON_1_RESULT(0x00),
    BUTTON_2_RESULT(0x01),
    DISPLAY(0x02),
    SUCCESS(0x03),
    FAILURE(0x04);
}

enum class OnBoardUserEventCommand(val rawValue: Int) {
    BUTTON_1_PRESSED(0x00),
    BUTTON_2_PRESSED(0x01),
    SUCCESS(0x02),
    FAILURE(0x03);

    val responseCommand: CommandReceiverCommand
        get() {
            return when (this) {
                BUTTON_1_PRESSED -> CommandReceiverCommand.BUTTON_1_RESULT
                BUTTON_2_PRESSED -> CommandReceiverCommand.BUTTON_2_RESULT
                else -> throw IllegalArgumentException("Invalid command: $this")
            }
        }
}