package com.riiiver.sample.qcc5100.util.extensions

import com.riiiver.sample.qcc5100.data.SlateFeature
import com.riiiver.sample.qcc5100.util.riiiver.piece_core.QCC5100TriggerPieceCore
import com.riiiver.sdk.ERApplet
import com.riiiver.sdk.ERBlockName
import com.riiiver.sdk.ERResult
import timber.log.Timber

fun ERApplet.isQCC5100Trigger(): Boolean {
    return getBlock(ERBlockName.TriggerBlock).metaData.executor == QCC5100TriggerPieceCore::class.java.simpleName
}

suspend fun ERApplet.isActivated(): Boolean {
    return when(val result = getActivateStatus()) {
        is ERResult.Success -> result.data
        is ERResult.Error -> {
            Timber.e("Error: ${result.error}")
            false
        }
    }
}

fun ERApplet.toSlateFeature(): SlateFeature {
    return object : SlateFeature {
        override val displayName: String = metaData.getTitle()
        override val id: String = metaData.id
    }
}

fun List<ERApplet>.kickedByQCC5100Trigger(): List<ERApplet> {
    return filter { it.isQCC5100Trigger() }
}

suspend fun List<ERApplet>.activatedApplets(): List<ERApplet> {
    return filter { it.isActivated() }
}