package com.riiiver.sample.qcc5100.util.firebase

import android.content.Context
import com.google.firebase.messaging.FirebaseMessaging
import com.google.firebase.messaging.FirebaseMessagingService
import com.google.firebase.messaging.RemoteMessage
import com.riiiver.sample.qcc5100.data.QCC5100Queue
import com.riiiver.sample.qcc5100.di.ApplicationScope
import com.riiiver.sample.qcc5100.util.ble.CommandReceiverCommand
import com.riiiver.sample.qcc5100.util.riiiver.IRiiiverSdkClient
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.launch
import timber.log.Timber
import javax.inject.Inject

@AndroidEntryPoint
class FcmService : FirebaseMessagingService() {
    // DI
    @Inject
    @ApplicationScope
    lateinit var externalScope: CoroutineScope
    @Inject lateinit var riiiverSdkClient: IRiiiverSdkClient

    override fun onCreate() {
        super.onCreate()
        Timber.d("onCreate")
        FirebaseMessaging.getInstance().token.addOnCompleteListener {
            if (it.isSuccessful) {
                val token = it.result
                Timber.d("token: $token")
                getSharedPreferences("_", MODE_PRIVATE).edit().putString("fb", token).apply()
                externalScope.launch {
                    riiiverSdkClient.setNotificationToken(token)
                }
            }
        }
    }

    override fun onNewToken(s: String) {
        super.onNewToken(s)
        Timber.d("newToken: $s")
        getSharedPreferences("_", MODE_PRIVATE).edit().putString("fb", s).apply()
        externalScope.launch {
            riiiverSdkClient.setNotificationToken(s)
        }
    }

    override fun onMessageReceived(remoteMessage: RemoteMessage) {
        super.onMessageReceived(remoteMessage)
        Timber.d("onMessageReceived: ${remoteMessage.data}")
        val notificationId = remoteMessage.data["notification_id"]
        notificationId?.let {
            if (it == "link") {
                Timber.i("## Remote push for executing an iiidea")
                QCC5100Queue.enqueue(CommandReceiverCommand.DISPLAY)
            }
            externalScope.launch {
                riiiverSdkClient.executeTaskForRemoteNotification(remoteMessage)
            }
        } ?: run {
            Timber.d("## Remote push does not have a notification_id (Riiiver SDK does not handle it)")
            // do something if needed
        }
    }

    companion object {
        fun getToken(context: Context): String? {
            return context.getSharedPreferences("_", MODE_PRIVATE).getString("fb", "empty")
        }
    }
}