package com.riiiver.sample.qcc5100.util.riiiver.piece_core

import com.riiiver.sdk.ERApplet
import com.riiiver.sdk.ERBlockName
import com.riiiver.sdk.delegates.ERBlockDelegate
import com.riiiver.sdk.delegates.ERBlockExecutor
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.SupervisorJob
import kotlinx.coroutines.launch
import timber.log.Timber
import kotlin.jvm.Throws

class QCC5100TriggerPieceCore: ERBlockExecutor {
    companion object {
        var externalScope: CoroutineScope? = null
        var delegates = mutableMapOf<String, ERBlockDelegate>()
            private set

        @Throws(IllegalStateException::class)
        fun didTrigger(appletId: String, input: Map<String, Any>?) {
            Timber.i("didTrigger is invoked. id = $appletId")

            if (delegates.isEmpty() || !delegates.containsKey(appletId)) {
                throw IllegalStateException("delegate for $appletId is missing.")
            }

            val output = mutableMapOf<String, Any>("triggerSampleInfo" to "")
            input?.get("output")?.let {
                output["triggerSampleInfo"] = it
            }

            externalScope?.launch {
                delegates[appletId]?.completed(output, null) ?: run {
                    throw IllegalStateException("delegate for $appletId is null.")
                }
            } ?: run {
                throw IllegalStateException("externalScope is null.")
            }
        }
    }

    init {
        externalScope =  CoroutineScope(SupervisorJob() + Dispatchers.IO)
    }

    override fun disable(applet: ERApplet) {
        Timber.i("disable is invoked(DEACTIVATED). ${applet.metaData.getTitle()}")
        delegates.remove(applet.metaData.id)
    }

    override fun enable(applet: ERApplet): Boolean {
        Timber.d("enable is invoked(ACTIVATE). ${applet.metaData.getTitle()}")
        return true
    }

    override suspend fun execute(
        applet: ERApplet,
        blockName: ERBlockName,
        preferenceJson: Map<String, Any>?,
        inputJson: Map<String, Any>?,
        handler: ERBlockDelegate
    ) {
        Timber.i("execute is invoked(ACTIVATED). ${applet.metaData.getTitle()}")
        delegates.putIfAbsent(applet.metaData.id, handler)
    }
}